Public Class ComputerCleaner
    Implements ICleaner

    Private Const _defaultProperties As String = "cn,description,operatingSystem,lastLogonTimestamp,modifyTimeStamp"
    Private Const _itemName As String = "computers"

#Region "Singleton Interface"

    Private Shared _accountCleaner As ComputerCleaner

    Public Shared Function GetInstance() As ComputerCleaner
        Try
            If _accountCleaner Is Nothing Then
                _accountCleaner = New ComputerCleaner
            End If
        Finally
        End Try

        Return _accountCleaner

    End Function

    'singleton pattern - cannot be constructed directly
    Private Sub New()

    End Sub

#End Region

    Public ReadOnly Property ItemName() As String Implements ICleaner.ItemName
        Get
            Return _itemName
        End Get
    End Property

    Public ReadOnly Property ExcludedGroupsFilter() As String Implements ICleaner.ExcludedGroupsFilter
        Get
            'TODO: make sure this loop works the same as the hard coded array list
            Dim sl As Collections.Specialized.StringCollection = _
                My.Settings.UserManagementAttributeCleanupExcludedGroups

            Dim filter As String = ""

            Dim s As String
            For Each s In sl
                filter &= "(!memberof=" & "CN=" & s & "," & My.Settings.LDAPOUDN & ")"
            Next

            Return filter
        End Get
    End Property

    Public ReadOnly Property StaleLogonDate() As Date Implements ICleaner.StaleLogonDate
        Get
            Return DateAdd(DateInterval.Day, (My.Settings.ComputerManagementStaleLogonDays * -1), System.DateTime.Now)
        End Get
    End Property

    Public ReadOnly Property StaleModifiedDate() As Date Implements ICleaner.StaleModifiedDate
        Get
            Return DateAdd(DateInterval.Day, (My.Settings.ComputerManagementStaleLogonDays * -1), System.DateTime.Now)
        End Get
    End Property

    Public Function getDeletionCandidate() As DataTable Implements ICleaner.getDeletionCandidate

        Dim myAD As ADComputerHelper = ADComputerHelper.GetInstance

        Dim filter As String = "(userAccountControl:1.2.840.113556.1.4.803:=2)"
        filter &= Me.ExcludedGroupsFilter

        Dim groupFilter As String
        If My.Settings.ComputerManagementTargetGroupCN.Length > 0 Then
            groupFilter = "CN=" & My.Settings.ComputerManagementTargetGroupCN & "," & My.Settings.LDAPOUDN
        Else
            groupFilter = ""
        End If

        Dim dt As DataTable
        dt = myAD.getComputers(myAD.LDAPRoot, "computer", groupFilter, _defaultProperties, filter)

        myAD.AddTranslatedTimeStamp(dt, "lastlogontimestamp", "LastLogonDate")
        myAD.AddTranslatedTimeString(dt, "modifytimestamp", "LastModifiedDate")

        'filter the dataset for dates
        Dim rowFilter As String
        'lastlogon date less than the stale date
        rowFilter = "ISNULL(LastLogonDate,'01/01/1900') < '" & Me.StaleLogonDate & "'"
        rowFilter &= " AND "
        'modified date less than the stale date
        rowFilter &= "ISNULL(LastModifiedDate,'01/01/1900') < '" & Me.StaleModifiedDate & "'"

        Dim dv As DataView = New DataView(dt)
        dv.RowFilter = rowFilter

        Return dv.ToTable

    End Function

    Public Function getDisableCandidates() As DataTable Implements ICleaner.getDisableCandidates

        Dim myAD As ADComputerHelper = ADComputerHelper.GetInstance

        'build the filter
        Dim filter As String
        'accounts not already disabled
        filter = "(!userAccountControl:1.2.840.113556.1.4.803:=2)"
        filter &= Me.ExcludedGroupsFilter

        Dim groupFilter As String
        If My.Settings.ComputerManagementTargetGroupCN.Length > 0 Then
            groupFilter = "CN=" & My.Settings.ComputerManagementTargetGroupCN & "," & My.Settings.LDAPOUDN
        Else
            groupFilter = ""
        End If

        Dim dt As DataTable
        dt = myAD.getComputers(myAD.LDAPRoot, "computer", groupFilter, _defaultProperties, filter)

        myAD.AddTranslatedTimeStamp(dt, "lastlogontimestamp", "LastLogonDate")
        myAD.AddTranslatedTimeString(dt, "modifytimestamp", "LastModifiedDate")

        'filter the dataset for dates
        Dim rowFilter As String
        'lastlogon date less than the stale date
        rowFilter = "ISNULL(LastLogonDate,'01/01/1900') < '" & Me.StaleLogonDate & "'"
        rowFilter &= " AND "
        'modified date less than the stale date
        rowFilter &= "ISNULL(LastModifiedDate,'01/01/1900') < '" & Me.StaleModifiedDate & "'"

        Dim dv As DataView = New DataView(dt)
        dv.RowFilter = rowFilter

        Return dv.ToTable

    End Function

    Public Function getStaleComputers() As DataTable Implements ICleaner.getStaleItems

        Dim myAD As ADComputerHelper = ADComputerHelper.GetInstance

        'build the filter
        Dim filter As String
        'accounts not already disabled
        filter = "(!userAccountControl:1.2.840.113556.1.4.803:=2)"
        'not already a member of the stale accounts group
        filter &= "(!memberof=" & "CN=" & My.Settings.ComputerManagementPendingDisableGroupCN & "," & My.Settings.LDAPOUDN & ")"
        filter &= Me.ExcludedGroupsFilter

        Dim groupFilter As String
        If My.Settings.ComputerManagementTargetGroupCN.Length > 0 Then
            groupFilter = "CN=" & My.Settings.ComputerManagementTargetGroupCN & "," & My.Settings.LDAPOUDN
        Else
            groupFilter = ""
        End If

        'get the target accounts
        Dim dt As DataTable = myAD.getComputers(myAD.LDAPRoot, "computer", groupFilter, _defaultProperties, filter)
        myAD.AddTranslatedTimeStamp(dt, "lastlogontimestamp", "LastLogonDate")
        myAD.AddTranslatedTimeString(dt, "modifytimestamp", "LastModifiedDate")

        'filter the dataset for dates
        Dim rowFilter As String
        'lastlogon date less than the stale date
        rowFilter = "ISNULL(LastLogonDate,'01/01/1900') < '" & Me.StaleLogonDate & "'"
        rowFilter &= " AND "
        'modified date less than the stale date
        rowFilter &= "ISNULL(LastModifiedDate,'01/01/1900') < '" & Me.StaleModifiedDate & "'"

        Dim dv As DataView = New DataView(dt)
        dv.RowFilter = rowFilter

        Return dv.ToTable

    End Function

    Public Sub MarkAsPendingDisable(ByVal ComputerCN As String, Optional ByVal LastLogonDate As Date = Nothing, _
        Optional ByVal LastModifiedDate As Date = Nothing) Implements ICleaner.MarkAsPendingDisable

        Dim myAD As ADComputerHelper = ADComputerHelper.GetInstance

        If Not myAD.IsMemberOfGroup(My.Settings.ComputerManagementPendingDisableGroupCN, ComputerCN) Then
            myAD.AddUserToGroup(My.Settings.ComputerManagementPendingDisableGroupCN, ComputerCN)

            Dim comment As String = DateTime.Today.ToString & " - identified as a stale login, marked as pending disable"
            If Not LastLogonDate = Nothing Then
                comment &= vbCrLf & "Last logon date of " & LastLogonDate.ToString
            End If
            comment &= vbCrLf & "Last modified " & LastModifiedDate.ToString

            myAD.AppendComment(ComputerCN, comment)
        End If

    End Sub

    Public Sub DisableComputer(ByVal ComputerCN As String, Optional ByVal LastLogonDate As Date = Nothing, _
        Optional ByVal LastModifiedDate As Date = Nothing) Implements ICleaner.DisableItem

        Dim myAD As ADComputerHelper = ADComputerHelper.GetInstance
        myAD.DisableAccount(ComputerCN)

        Dim comment As String = DateTime.Today.ToString & " - disabled user account, previously identified as a stale login and marked as pending disable"
        If Not LastLogonDate = Nothing Then
            comment &= vbCrLf & "Last logon date of " & LastLogonDate.ToString
        Else
            comment &= vbCrLf & "Last logon date not present"
        End If
        comment &= vbCrLf & "Last modified " & LastModifiedDate.ToString

        myAD.AppendComment(ComputerCN, comment)

    End Sub

End Class
